//
//  Fish.swift
//  NyanFat
//
//  Created by 131e55 on 1/24/15.
//  Copyright (c) 2015 GGJ2015Team11. All rights reserved.
//

import SpriteKit

let sharedFishList: [[String: NSString]] = [
    ["score": "1", "moveInterval": "0.5"],
    ["score": "10", "moveInterval": "0.5"],
    ["score": "50", "moveInterval": "0.5"],
    ["score": "100", "moveInterval": "0.5"],
    ["score": "500", "moveInterval": "0.5"]
]

protocol FishDelegate {
    func fishDead(score: Int, tag: Int)
}

class Fish: SKSpriteNode {
    var delegate: FishDelegate?
    private var _id: Int!
    private var _score: Int!
    private var _moveInterval: NSTimeInterval!
    private var _direction: String!             // "right" or "left"
    private var _touched = false
    private var _moveUpDownAction: SKAction!
    private var _moveX: SKAction!
    var tag: Int!
    var pasued = false

    override init(texture: SKTexture!, color: UIColor!, size: CGSize) {
        super.init(texture: texture, color: SKColor.clearColor(), size: size)
    }

    convenience init(id: Int) {
        let imageNamed = "fish_" + String(id)
        let texture = SKTexture(imageNamed: imageNamed)
        let size = texture.size()

        self.init(texture: texture, color: SKColor.clearColor(), size: size)
        self.userInteractionEnabled = true
        self._id = id
        self._score = sharedFishList[id]["score"]!.integerValue
        self._moveInterval = sharedFishList[id]["moveInterval"]!.doubleValue

        var x: CGFloat = 0
        var y: CGFloat = 0

        switch arc4random_uniform(3) {
        case 0: // 右から
            self._direction = "left"
            x = gScreenSize.width + self.size.width
            y = CGFloat(arc4random_uniform(UInt32(gScreenSize.height)))
        case 1: // 左から
            self._direction = "right"
            self.xScale = -1
            x = -self.size.width
            y = CGFloat(arc4random_uniform(UInt32(gScreenSize.height)))
        case 2: // 下から
            x = CGFloat(arc4random_uniform(UInt32(gScreenSize.width)))
            if x < gScreenSize.width/2 {
                self._direction = "right"
                self.xScale = -1
            }
            else {
                self._direction = "left"
            }
            y = -self.size.height
        default:
            break
        }

        // 位置決め
        self.position = CGPoint(
            x: x,
            y: y  // ぴったり画面の下に隠れる位置
        )

        // 上下にふよふよ
        let moveUp = SKAction.moveByX(0, y: 10, duration: 1.0)
        let moveDown = SKAction.moveByX(0, y: -10, duration: 1.0)
        moveUp.timingMode = SKActionTimingMode.EaseInEaseOut;
        moveDown.timingMode = SKActionTimingMode.EaseInEaseOut;
        self._moveUpDownAction = SKAction.repeatActionForever(SKAction.sequence([moveUp, moveDown]))
        self.runAction(self._moveUpDownAction)

        // 魚自身の移動
        let deltaX = (self._direction == "right") ? 4 : -4
        self._moveX = SKAction.repeatActionForever(SKAction.moveByX(CGFloat(deltaX), y: 8, duration: self._moveInterval))
        self.runAction(self._moveX)
    }

    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func touchesBegan(touches: NSSet, withEvent event: UIEvent) {
        println("touch")
        if !self.paused && !self._touched {
            let moveCenter = SKAction.moveTo(
                CGPoint(
                    x: gCenterPoint.x,
                    y: gCenterPoint.y + CGFloat(sharedCurrentCatMouthDistance)
                ), duration: 1.5)
            moveCenter.timingMode = SKActionTimingMode.EaseIn
            let scaleDownY = SKAction.scaleYTo(0.25, duration: 1.5)
            let scaleDownX = SKAction.scaleXTo(0.25 * self.xScale, duration: 1.5)
            self.runAction(SKAction.sequence([
                SKAction.group([moveCenter, scaleDownX, scaleDownY]),
                SKAction.runBlock({
                    self.delegate?.fishDead(self._score, tag: self.tag)
                    return
                })
            ]))
            self._touched = true
        }
    }

    func actionStart() {
        self.runAction(self._moveUpDownAction)
        self.runAction(self._moveX)
    }

    func actionStop() {
        self.removeAllActions()
    }

    func gameEndAction() {
        let moveY = SKAction.moveToY(gScreenSize.height + self.size.height, duration: 1.0)
        moveY.timingMode = SKActionTimingMode.EaseInEaseOut;
        self.runAction(moveY)
    }
}