//
//  GameScene.swift
//  NyanFat
//
//  Created by 131e55 on 1/24/15.
//  Copyright (c) 2015 GGJ2015Team11. All rights reserved.
//

import SpriteKit

protocol GameSceneDelegate {
    func gameEnd(Int)
}

class GameScene: SKScene, FishDelegate {
    var gameDelegate: GameSceneDelegate?
    private var _gameTimeInitialized = false
    private var _gameTime: NSTimeInterval = 0
    private let _limitTime: NSTimeInterval = 60.0
    private var _countDown: NSTimeInterval = 60.0 // 現在のゲームの残り時間[ms]
    private var _lastTime: NSTimeInterval = 0  // ゲーム開始時刻
    private var _countDownStopped = true       // カウントダウンが停止中か(Fever||Pause)
    private var _speed: CGFloat = 0.1           // 画面のスクロールスピード(猫の落下の速さ)[px/frame]
    private var _fish = [Fish]()                // 生成された全ての魚
    private var _fishCnt = 0                    // 今までに生成された魚の総数。
    private let _cat = Cat()
    private let _backgrounds = [
        SKSpriteNode(imageNamed: "background_0"),
        SKSpriteNode(imageNamed: "background_1"),
        SKSpriteNode(imageNamed: "background_2"),
        SKSpriteNode(imageNamed: "background_3")]
    private var _frameCnt = 0
    private var _feverGauge = 0
    private let _feverNum = 30                  // 何匹の魚を食べたらフィーバー可能になるか
    private var _fishCycle = 50                 // 何フレーム毎に魚を生成するか
    private let _pauseButton = UIButton(frame: CGRect(x: 0, y: 0, width: 60, height: 60))
    private var _area = 0
    private let _scoreNumbers = [
        SKSpriteNode(imageNamed: "number_0"),
        SKSpriteNode(imageNamed: "number_0"),
        SKSpriteNode(imageNamed: "number_0"),
        SKSpriteNode(imageNamed: "number_0")
    ]
    private let _scoreLabel = SKSpriteNode(imageNamed: "score")
    private let _timeIcon = SKSpriteNode(imageNamed: "watch")
    private let _timeNumbers = [
        SKSpriteNode(imageNamed: "number_0"),
        SKSpriteNode(imageNamed: "number_0")
    ]

    override init() {
        super.init(size: gScreenSize)
    }

    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func didMoveToView(view: SKView) {
        var bgHeight: CGFloat = 0
        for bg in self._backgrounds {
            bg.position = gCenterPoint
            bg.runAction(SKAction.moveByX(0, y: bgHeight, duration: 0))
            bgHeight -= bg.size.height
            self.addChild(bg)
        }

        // 猫
        self._cat.position = CGPoint(
            x: gCenterPoint.x,
            y: self._cat.size.height/2
        )
        self._cat.runAction(SKAction.rotateToAngle(CGFloat(3.14), duration: 0))
        self._cat.hidden = true
        self.addChild(self._cat)

        // Update: ポーズボタン
//        var image = UIImage(named: "pause")
//        self._pauseButton.setImage(image, forState: .Normal)
//        self._pauseButton.layer.position = CGPoint(
//            x: gScreenSize.width - self._pauseButton.frame.size.width / 2 - 16,
//            y: gScreenSize.height - self._pauseButton.frame.size.height / 2 - 16
//        )
//        self._pauseButton.addTarget(self, action: "pauseTouched", forControlEvents: .TouchUpInside)
//        self.view?.addSubview(self._pauseButton)

        // フィーバーボタン
//        image = UIImage(named: "pause")
//        let feverButton = UIButton(frame: CGRect(x:0, y: 0, width: 40, height: 40))
//        feverButton.setImage(image, forState: .Normal)
//        feverButton.addTarget(self, action: "feverTouched", forControlEvents: .TouchUpInside)
//        self.view?.addSubview(feverButton)

        var index = 0
        for number in self._scoreNumbers {
            number.position = CGPoint(
                x: gScreenSize.width - number.size.width/2 - number.size.width * CGFloat(index) - 8,
                y: gScreenSize.height - number.size.height / 2
            )
            index++
            number.zPosition = 100
            self.addChild(number)
        }
        self._scoreLabel.position = CGPoint(
            x: self._scoreNumbers[3].position.x - self._scoreLabel.size.width/2 - 8,
            y: self._scoreNumbers[3].position.y
        )
        self._scoreLabel.zPosition = 100
        self.addChild(self._scoreLabel)

        self._timeIcon.position = CGPoint(
            x: self._timeIcon.size.width / 2 + 8,
            y: gScreenSize.height - self._timeIcon.size.height / 2
        )
        self._timeIcon.zPosition = 100
        self.addChild(self._timeIcon)

        index = 0
        for number in self._timeNumbers {
            number.position = CGPoint(
                x: self._timeIcon.position.x + self._timeIcon.size.width/2 + number.size.width - number.size.width * CGFloat(index),
                y: gScreenSize.height - number.size.height / 2
            )
            index++
            number.zPosition = 100
            self.addChild(number)
        }

        self.gameStart()
    }

    func gameStart() {
        // ゲームスタートアニメーション
        var nyanATextures = [SKTexture]()
        for i in 0...12 {
            nyanATextures.append(SKTexture(imageNamed: "A_" + String(i)))
        }
        var nyanBTextures = [SKTexture]()
        nyanBTextures.append(SKTexture(imageNamed: "B_0"))
        var nyanCTextures = [SKTexture]()
        for i in 0...4 {
            nyanCTextures.append(SKTexture(imageNamed: "C_" + String(i)))
        }
        var bgAction = SKAction.moveByX(0, y: self._backgrounds[0].size.height * 3, duration: 60)
        let nyan = SKSpriteNode(texture: nyanATextures[0])
        nyan.position = gCenterPoint
        self.addChild(nyan)
        nyan.runAction(SKAction.sequence([
            SKAction.animateWithTextures(nyanATextures, timePerFrame: 0.25),
            SKAction.animateWithTextures(nyanBTextures, timePerFrame: 0.75),
            SKAction.animateWithTextures(nyanCTextures, timePerFrame: 0.25),
            SKAction.runBlock({
                for bg in self._backgrounds {
                    bg.runAction(SKAction.sequence([
                        SKAction.runBlock({
                            nyan.hidden = true
                            self._cat.hidden = false
                            let down = SKAction.moveToY(-self._cat.size.height/2, duration: 5)
                            down.timingMode = SKActionTimingMode.EaseInEaseOut
                            self._cat.runAction(SKAction.sequence([
                                down,
                                SKAction.moveToY(gCenterPoint.y, duration: 12),
                                SKAction.rotateToAngle(0, duration: 1)
                            ]))
                            self._countDownStopped = false
                        }),
                        bgAction
                    ]))
                }
            }),
        ]))
//        var bgHeight: CGFloat = 0
//        for bg in self._backgrounds {
//            bg.position = gCenterPoint
//            bg.runAction(SKAction.moveByX(0, y: bgHeight, duration: 0))
//            bgHeight -= bg.size.height
//        }
//        self._cat.initialize()
//        self._timeNumbers[0].texture = SKTexture(imageNamed: "number_0")
//        self._timeNumbers[1].texture = SKTexture(imageNamed: "number_0")
//        for score in self._scoreNumbers {
//            score.texture = SKTexture(imageNamed: "number_0")
//        }
//        self._fishCycle = 60
//        self._area = 0
//        self._gameTimeInitialized = false
//        self._countDownStopped = false
//        self._countDown = self._limitTime
//        self._fishCnt = 0
    }

    override func update(currentTime: CFTimeInterval) {
        // カウントダウンが止まっていないなら(フィーバー中・ポーズ中でない)
        if !self._countDownStopped {
            self._frameCnt++

            // 残り時間の更新
            if !self._gameTimeInitialized {
                self._gameTimeInitialized = true;
                self._lastTime = currentTime;
            }
            let timeSinceLast = currentTime - self._lastTime;
            self._lastTime = currentTime
            self._gameTime += timeSinceLast
            self._countDown = self._limitTime - self._gameTime

            var num = Int(floor(Float(self._countDown))) % 10
            self._timeNumbers[0].texture = SKTexture(imageNamed: "number_" + String(num))
            num = Int(floor(Float(self._countDown))) / 10
            self._timeNumbers[1].texture = SKTexture(imageNamed: "number_" + String(num))

//            println(self._countDown)
            if self._countDown <= 0 {
                self._pauseButton.hidden = true
                self._countDownStopped = true
                self._backgrounds[3].runAction(SKAction.moveToY(gCenterPoint.y, duration: 0))
                self._cat.runAction(
                    SKAction.sequence([
                        SKAction.moveToY(128 + self._cat.size.height/2, duration: 3.0),
                        SKAction.runBlock({
                            self.gameDelegate?.gameEnd(self._cat.score)
                            return
                        })
                    ])
                )
                for fish in self._fish {
                    fish.userInteractionEnabled = false
                    fish.gameEndAction()
                }
                self._timeNumbers[0].texture = SKTexture(imageNamed: "number_0")
                self._timeNumbers[1].texture = SKTexture(imageNamed: "number_0")
            }

            // 魚を一定間隔で生成
            if self._frameCnt >= self._fishCycle {
                var range = 3
                switch self._area {
                case 0: range = 0
                case 1: range = 3
                case 2: range = 4
                case 3: range = 5
                default: break
                }
                if range > 0 {
                    var fish = Fish(id: Int(arc4random_uniform(UInt32(range))))
                    fish.delegate = self
                    fish.tag = self._fishCnt
                    self._fish.append(fish)
                    self.addChild(fish)
                    self._fishCnt++
                    self._frameCnt = 0
                    // 時間が経つにつれ魚をでやすくする
                    if (self._fishCycle > 25) {
                        self._fishCycle -= 1
                    }
                }
            }

            // ランクを決定
            var index = 0
            for bg in self._backgrounds {
                if bg.position.y >= 0 {
                    self._area = index
                }
                index++
            }
        }
    }

    func fishDead(score: Int, tag: Int) {
        self._cat.addScore(score)
        if self._cat.score > 9999 {
            self._cat.score = 9999
        }

        var num = (self._cat.score % 10000) / 1000
        self._scoreNumbers[3].texture = SKTexture(imageNamed: "number_" + String(num))
        num = (self._cat.score % 1000) / 100
        self._scoreNumbers[2].texture = SKTexture(imageNamed: "number_" + String(num))
        num = (self._cat.score % 100) / 10
        self._scoreNumbers[1].texture = SKTexture(imageNamed: "number_" + String(num))
        num = self._cat.score % 10
        self._scoreNumbers[0].texture = SKTexture(imageNamed: "number_" + String(num))

        self._feverGauge++;

        var index = 0
        for fish in self._fish {
            if fish.tag == tag {
                fish.removeFromParent()
                self._fish.removeAtIndex(index)
            }
            index++
        }
    }

    func pauseTouched() {
        self._countDownStopped = !self._countDownStopped
        if !self._countDownStopped {
            self._gameTimeInitialized = false
            for fish in self._fish {
                fish.actionStart()
                fish.paused = false
            }
        }
        else {
            for fish in self._fish {
                fish.actionStop()
                fish.paused = true
            }
        }
    }

    func feverTouched() {
//        println("fever!")
    }
}
